@@ -12,14 +12,14 @@ use constant OS_UNIX => "Net::DNS::Resolver::UNIX";
 use constant OS_CONF => grep eval "require $_", OS_SPEC, OS_UNIX;	## no critic
 use base (OS_CONF)[0];			## backward compatibility only
 
-
 our $VERSION;
 
 BEGIN {
-	$VERSION = '1.31_03';
+	$VERSION = '1.35_01';
 	eval { __PACKAGE__->bootstrap($VERSION) };
 }
 
+
 use constant UB_VERSION => scalar eval {
 	my $version = Net::DNS::Resolver::libunbound->VERSION();
 	my ( $major, $minor, $minimus ) = split /\D/, $version;
@@ -180,23 +180,29 @@ sub new {
 		my $value = shift @args;
 		$self->$attr( ref($value) ? @$value : $value );
 	}
-	$self->_finalise_config;				# force context rebuild
+	$self->_finalise_config;				# merge config updates
 	return $self;
 }
 
 
 =head2 nameservers
 
-	my $dnssec_resolver = Net::DNS::Resolver::Unbound->new(
+	my $DNSSEC_resolver = Net::DNS::Resolver::Unbound->new(
 		nameservers => [],	# override /etc/resolv.conf
 		add_ta_file => '/var/lib/unbound/root.key'
 		);
 
+	my $DNSSEC_stub = Net::DNS::Resolver::Unbound->new(
+		nameservers => ['8.8.8.8', '1.1.1.1'],
+		add_ta_file => '/var/lib/unbound/root.key'
+		);
+
 	my $DoT_resolver = Net::DNS::Resolver::Unbound->new(
 		nameserver => '2606:4700:4700::1111@853#cloudflare-dns.com',
 		nameserver => '1.1.1.1@853#cloudflare-dns.com',
 		nameserver => '2001:4860:4860::8888@853#dns.google',
 		nameserver => '8.8.8.8@853#dns.google',
+		add_ta_file => '/var/lib/unbound/root.key',
 		option	=> ['tls-cert-bundle' => '/etc/ssl/cert.pem'],
 		set_tls	=> 1
 		);
@@ -205,7 +211,7 @@ sub new {
 	@nameservers = $resolver->nameservers;
 
 By default, DNS queries are sent to the IP addresses listed in
-/etc/resolv.conf or similar platform-specific sources.
+F</etc/resolv.conf> or similar platform-specific sources.
 
 =cut
 
@@ -238,31 +244,33 @@ use constant UB_SEND => UB_CONTEXT->can('ub_send');
 sub send {
 	my ( $self, @argument ) = @_;
 
-	my ($packet) = @argument;
-	if ( ref $packet ) {					# resolve packets asynchronously
-		my $handle = $self->bgsend(@argument);
-		return $self->bgread($handle);
-	}
-
-	$self->_finalise_config;
-	$self->_reset_errorstring;
-
-	my $query  = $self->_make_query_packet(@argument);
-	my ($q)	   = $query->question;
-	my $result = $self->{ub_ctx}->ub_resolve( $q->name, $q->{qtype}, $q->{qclass} );
-	return $self->_decode_result($result);
+	my $handle = $self->bgsend(@argument);			# resolve asynchronously
+	return $self->bgread($handle);
 }
 
 sub bgsend {
 	my ( $self, @argument ) = @_;
-	$self->_finalise_config;
 	$self->_reset_errorstring;
+	$self->_finalise_config;				# merge config updates
 
 	my $query = $self->_make_query_packet(@argument);
-	my $image = $query->encode;
-	my $ident = $query->header->id;
-	my ($q)	  = $query->question;
-	return $self->{ub_ctx}->ub_resolve_async( $q->name, $q->{qtype}, $q->{qclass}, $ident );
+	my ($q) = $query->question;
+	return $self->{ub_ctx}->ub_resolve_async( $q->name, $q->{qtype}, $q->{qclass}, $query );
+}
+
+sub bgread {
+	my ( $self, $handle ) = @_;
+	return unless $handle;
+
+	$self->{ub_ctx}->ub_wait;
+	$self->errorstring( $handle->err );
+	my $qident = $handle->async_id;
+
+	my $reply = $self->_decode_result( $handle->result ) || return;
+	$handle->query->{id} = $reply->_quid;			# zero id replaced by random lie
+
+	$reply->print if $self->debug;
+	return $reply;
 }
 
 sub bgbusy {
@@ -274,18 +282,6 @@ sub bgbusy {
 	return $handle->waiting;
 }
 
-sub bgread {
-	my ( $self, $handle ) = @_;
-	return unless $handle;
-
-	$self->{ub_ctx}->ub_wait;
-	$self->errorstring( $handle->err );
-
-	my $reply = $self->_decode_result( $handle->result ) || return;
-	$reply->header->id( $handle->query_id );		# lying toad!;
-	return $reply;
-}
-
 
 =head2 option
 
@@ -372,7 +368,7 @@ sub set_stub {
 Extract nameserver list from resolv.conf(5) format configuration file.
 Any domain, searchlist, ndots or other settings are ignored.
 
-Note that Net::DNS builds its own nameserver list using /etc/resolv.conf
+Note that Net::DNS builds its own nameserver list using F</etc/resolv.conf>
 or other platform-specific sources.
 
 =cut
@@ -387,7 +383,7 @@ sub resolv_conf {
 
 	$resolver->hosts( 'filename' );
 
-Read list of hosts from the filename given, usually "/etc/hosts".
+Read list of hosts from the filename given, usually F</etc/hosts>.
 These addresses are not flagged as DNSSEC secure when queried.
 
 =cut
@@ -531,7 +527,7 @@ sub string {
 ;; ${prefer}	$self->{$prefer}	ndots	$self->{ndots}
 ;; ${force}	$self->{$force}	debug	$self->{debug}
 END
-	$self->_finalise_config;				# force config rebuild
+	$self->_finalise_config;				# merge config updates
 	my %config = %{$self->{ub_cfg}};
 	my $optref = $config{set_option} || [];
 	my @option = @$optref;					# pre-sorted option list
@@ -573,8 +569,7 @@ sub _decode_result {
 	my $buffer = $result->answer_packet || return;
 	my $packet = Net::DNS::Packet->decode( \$buffer );
 	$self->errorstring($@);
-	$packet->print if $self->debug;
-
+	delete $packet->{id} unless $packet->{id};		# discard zero id
 	return $packet;
 }
 
